@@ -250,7 +250,6 @@ process_line__common (char     *line,
 		double fraction = (double) comm->n_file++ / comm->n_files;
 		fr_command_progress (comm, fraction);
 	}
-
 }
 
 
@@ -483,6 +482,11 @@ fr_command_zip_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
+	if ((!is_program_in_path("zip")) ||
+	    (!is_program_in_path("unzip"))) {
+		return NULL;
+	}
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ZIP, NULL));
 	fr_command_construct (comm, process, filename);
 
